@@ -1,6 +1,6 @@
 /*****************************************************************************
- * Secure Locate v1.0                                                        *
- * October 8, 1998                                                           *
+ * Secure Locate v1.1                                                        *
+ * October 9, 1998                                                           *
  * Programmed by: Kevin Lindsay                                              *
  * Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.          *
  *                                                                           *
@@ -48,12 +48,13 @@
 
 /* GLOBALS */
 
-#define VERSION "Secure Locate v1.0 - Released October 8, 1998\n"
+#define VERSION "Secure Locate v1.1 - Released October 9, 1998\n"
 #define SLOCATEDB "/var/lib/slocate/slocate.db"
 #define TMPSLOCATEDB "/var/lib/slocate/slocate.tmp"
 #define GRPFILE "/etc/group"
 #define SLOC_ESC 0x80
 #define SLOC_GRP "slocate"
+#define SLOC_UID 0
 
 char *EXCLUDE_DIR;
 int EXCLUDE=0;
@@ -67,33 +68,30 @@ gid_t GID;
 int first=1;
 char slevel='1';
 unsigned short SLOC_GID=65534; /* Seems to be the default nobody gid */
-unsigned short tuid = 65535;
-unsigned short tgid = 65535;
-unsigned short tmode = 65535;
 
 /* Usage */
 
 void
 usage() 
 {
-	printf("Secure Locate v1.0\n"
-			 "Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.\n"
-			 "\n"
-			 "usage: %s [options] <search string>\n\n"
-			 "   Options:\n"
-			 "   -u                 - Create slocate database starting at path /.\n"
-			 "   -U <dir>           - Create slocate database starting at path <dir>.\n"			 
-			 "   -e <dir1,dir2,...> - Exclude directories from slocate database.\n"
-			 "   -l <[01]>          - Security level. \n"
-			 "                           0 turns security checks off. This will make\n"
+   printf("Secure Locate v1.1\n"
+          "Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.\n"
+          "\n"
+          "usage: %s [options] <search string>\n\n"
+          "   Options:\n"
+          "   -u                 - Create slocate database starting at path /.\n"
+          "   -U <dir>           - Create slocate database starting at path <dir>.\n"          
+          "   -e <dir1,dir2,...> - Exclude directories from slocate database.\n"
+          "   -l <[01]>          - Security level. \n"
+          "                           0 turns security checks off. This will make\n"
           "                             searchs faster.\n"
-			 "                           1 turns security checks on. This is the default.\n"
-			 "\n"
-			 "Author: Kevin Lindsay\n"
-			 "Bugs: klindsay@mkintraweb.com\n"
-			 "\n",progname);
-	
-	exit(0);
+          "                           1 turns security checks on. This is the default.\n"
+          "\n"
+          "Author: Kevin Lindsay\n"
+          "Bugs: klindsay@mkintraweb.com\n"
+          "\n",progname);
+   
+   exit(0);
 }
 
 /* My fchdir() so that I can do cleaner error checking */
@@ -121,34 +119,17 @@ MYchdir(char *filepath, char *func)
 static void
 put_short (int c, FILE *fp)
 {
-	putc (c >> 8, fp);
-	putc (c, fp);
+   putc (c >> 8, fp);
+   putc (c, fp);
 }
 
 static int
 get_short (FILE *fp)
 {
-	register short x;
-	
-	x = fgetc (fp);
-	return (x << 8) | (fgetc (fp) & 0xff);
-}
-
-static void
-Uput_short (unsigned short c, FILE *fp)
-{
-	putc (c >> 8, fp);
-	putc (c, fp);
-}
-
-static int
-Uget_short (FILE *fp)
-{
-	register unsigned short x;
-	
-	x = fgetc (fp);
-	
-	return (x << 8) | (fgetc (fp) & 0xff);
+   register short x;
+   
+   x = fgetc (fp);
+   return (x << 8) | (fgetc (fp) & 0xff);
 }
 
 /* Get the GID for group slocate */
@@ -156,71 +137,71 @@ Uget_short (FILE *fp)
 unsigned short
 get_gid()
 {
-	unsigned short GGID=SLOC_GID;
-	FILE *fd;
-	int ch;
-	unsigned int grow=0;
-	char *item;
-	int foundit=0;
-	
-	fd = fopen(GRPFILE,"r");
-	
-	if (fd == NULL) {
-		fprintf(stderr,"%s: Could not open %s for reading.\n\n"
-				  "Read the README.DOC file in the Secure Locate source directory.\n",progname,GRPFILE);
-		exit(1);
-	}
-	
-	item = (char *)malloc(1);
-	
-	while((ch = getc(fd)) != EOF) {
-		grow++;
-		item = (char *)realloc(item,grow);
-		item[grow-1] = ch;
-		if (ch == ':') {
-			item[grow-1] = '\0';
-//			printf("Item: %s\nSLOC: %s\n",item,SLOC_GRP);
-			if (!strcmp(item,SLOC_GRP)) {
-				
-				while ((ch = getc(fd)) != ':' && ch != EOF);
-				if (ch == EOF) 
-					 break;
-				grow=0;
-				free(item);
-				item = (char *)malloc(1);
-				while ((ch = getc(fd)) != ':' && ch != EOF) {
-					grow++;
-					item = (char *)realloc(item,grow);
-					item[grow-1] = ch;
-				}
-				if (ch == EOF)
-					 break;
-				
-				grow++;
-				item = (char *)realloc(item,grow);
-				item[grow-1] = '\0';				
-				GGID = atoi(item);
-				free(item);
-				foundit=1;				
-				break;
-			} else {
-				while ((ch = getc(fd)) != '\n' && ch != EOF);
-				if (ch == EOF)
-					 break;
-				grow=0;
-			}
-		}
-	}
-	
-	fclose(fd);
-	
-	if (!foundit) {
-		fprintf(stderr,"%s: Could not find the group: %s in the: %s file.\n",progname,SLOC_GRP,GRPFILE);
-		fprintf(stderr,"This is a result of the group missing or a corrupted group file.\n");
-		exit(1);
-	}
-	
-	return GGID;
+   unsigned short GGID=SLOC_GID;
+   FILE *fd;
+   int ch;
+   unsigned int grow=0;
+   char *item;
+   int foundit=0;
+   
+   fd = fopen(GRPFILE,"r");
+   
+   if (fd == NULL) {
+      fprintf(stderr,"%s: Could not open %s for reading.\n\n"
+              "Read the README.DOC file in the Secure Locate source directory.\n",progname,GRPFILE);
+      exit(1);
+   }
+   
+   item = (char *)malloc(1);
+   
+   while((ch = getc(fd)) != EOF) {
+      grow++;
+      item = (char *)realloc(item,grow);
+      item[grow-1] = ch;
+      if (ch == ':') {
+         item[grow-1] = '\0';
+
+         if (!strcmp(item,SLOC_GRP)) {
+            
+            while ((ch = getc(fd)) != ':' && ch != EOF);
+            if (ch == EOF) 
+                break;
+            grow=0;
+            free(item);
+            item = (char *)malloc(1);
+            while ((ch = getc(fd)) != ':' && ch != EOF) {
+               grow++;
+               item = (char *)realloc(item,grow);
+               item[grow-1] = ch;
+            }
+            if (ch == EOF)
+                break;
+            
+            grow++;
+            item = (char *)realloc(item,grow);
+            item[grow-1] = '\0';            
+            GGID = atoi(item);
+            free(item);
+            foundit=1;            
+            break;
+         } else {
+            while ((ch = getc(fd)) != '\n' && ch != EOF);
+            if (ch == EOF)
+                break;
+            grow=0;
+         }
+      }
+   }
+   
+   fclose(fd);
+   
+   if (!foundit) {
+      fprintf(stderr,"%s: Could not find the group: %s in the: %s file.\n",progname,SLOC_GRP,GRPFILE);
+      fprintf(stderr,"This is a result of the group missing or a corrupted group file.\n");
+      exit(1);
+   }
+   
+   return GGID;
 }
 
 /* Parse Exclude Command */
@@ -228,58 +209,58 @@ get_gid()
 char *
 parse_exclude(char *estr)
 {
-	char *excludestr;
-	char *ptr1;
-	char *ptr2=NULL;
-	int grow=1;
-	int elen;
-	
-	EXCLUDE=1;
+   char *excludestr;
+   char *ptr1;
+   char *ptr2=NULL;
+   int grow=1;
+   int elen;
+   
+   EXCLUDE=1;
 
-	excludestr = (char *)malloc(1);
-	excludestr[0] = '\0';
-	
-	elen = strlen(estr);
+   excludestr = (char *)malloc(1);
+   excludestr[0] = '\0';
+   
+   elen = strlen(estr);
 
-	ptr1 = estr;
-	ptr2 = ptr1;
+   ptr1 = estr;
+   ptr2 = ptr1;
 
-	while (ptr2[0] != '\0')  {
+   while (ptr2[0] != '\0')  {
 
-		if ((ptr2 = strchr(ptr1,',')) == NULL) {
-			ptr2 = strchr(ptr1,'\0');
-		}
+      if ((ptr2 = strchr(ptr1,',')) == NULL) {
+         ptr2 = strchr(ptr1,'\0');
+      }
 
-		grow += (ptr2-ptr1)+2;
-		excludestr = (char *)realloc(excludestr,grow);
-		strcat(excludestr,"*");
-		strncat(excludestr,ptr1,ptr2-ptr1);
-		strcat(excludestr,"*");
-		if (ptr2[0] == ',')
-			 ptr1 = ptr2+1;
-	}
-		
-	return (excludestr);
+      grow += (ptr2-ptr1)+2;
+      excludestr = (char *)realloc(excludestr,grow);
+      strcat(excludestr,"*");
+      strncat(excludestr,ptr1,ptr2-ptr1);
+      strcat(excludestr,"*");
+      if (ptr2[0] == ',')
+          ptr1 = ptr2+1;
+   }
+      
+   return (excludestr);
 }
 
 int
 match_exclude(char *path,char *filedir) {
-	int res=0;
-	char *newstr;
-	
-	newstr = (char *)statmalloc(strlen(path)+strlen(filedir)+3);
-	newstr[0] = '\0';
-	strcat(newstr,"*");
-	strcat(newstr,path);
-	strcat(newstr,filedir);	
-	strcat(newstr,"*");
-	
-	if (strstr(EXCLUDE_DIR,newstr) != NULL)		 
-		 res=1;
-	
-	freemalloc(newstr);
-	
-	return res;
+   int res=0;
+   char *newstr;
+   
+   newstr = (char *)statmalloc(strlen(path)+strlen(filedir)+3);
+   newstr[0] = '\0';
+   strcat(newstr,"*");
+   strcat(newstr,path);
+   strcat(newstr,filedir);   
+   strcat(newstr,"*");
+   
+   if (strstr(EXCLUDE_DIR,newstr) != NULL)       
+       res=1;
+   
+   freemalloc(newstr);
+   
+   return res;
 }
 
 /* Load Dir Contents into Linked List */
@@ -340,14 +321,13 @@ load_dir(int dir_fd, dir_item *listptr)
          listptr = listptr->down;
       }
       
-//      printf("LD: %s\n",filename->d_name);
       
       /* Add information from filenames->d_name to listptr */
       listptr->name = (char *)statmalloc(strlen(filename->d_name)+1);
       listptr->name[0] = '\0';
       strcat(listptr->name,filename->d_name);
       listptr->name_len = strlen(listptr->name);      
-//      printf("Name2: %s\n",listptr->name);
+
       if (lstat(filename->d_name,&statres) == -1) {
          fprintf(stderr,"%s: lstat: %s: %s\n",progname,filename->d_name,strerror(errno));
       }
@@ -384,7 +364,7 @@ get_cur_dir(dir_item *listptr, char *dirstr)
    }
 
    dir_ptr = listptr;
-	
+   
    while (!dir_ptr->root) {
       dir_ptr=dir_ptr->left;
       items++;
@@ -398,20 +378,20 @@ get_cur_dir(dir_item *listptr, char *dirstr)
       dirarray[i] = (char *)malloc(strlen(dir_ptr->name)+1);
       dirarray[i][0] = '\0';
       strcat(dirarray[i],dir_ptr->name);
-//		printf("Name: *%s*\n",dir_ptr->name);
+
       dir_ptr=dir_ptr->left;
    }
 
    cur_dir = (char *) malloc(1);
    cur_dir[0] = '\0';
-	grow=1;
+   grow=1;
    
    for (i = 0; i != items; i++) {
       grow+=strlen(dirarray[i])+1;
       cur_dir = (char *)realloc(cur_dir,grow);
       strcat(cur_dir,dirarray[i]);
-		if ((strcmp(dirstr,"/") != 0 && i == 0) || i > 0)
-			 strcat(cur_dir,"/");
+      if ((strcmp(dirstr,"/") != 0 && i == 0) || i > 0)
+          strcat(cur_dir,"/");
    }
 
    for (i = 0; i != items; i++) {
@@ -421,8 +401,6 @@ get_cur_dir(dir_item *listptr, char *dirstr)
    free(dirarray);
 
    
-//   printf("Cur: %s\n",cur_dir);   
-
    return(cur_dir);
 }
 
@@ -436,105 +414,73 @@ frcode(char *dir_path, char *filename, struct stat statres)
    char *new_path;
    char *ptr1;
    char *ptr2;
-	unsigned short suid;
-	unsigned short sgid;
-	unsigned short smode;
+
    int i;
-	FILE *fd;	
-	
+   FILE *fd;   
+   
    cur_path = (char *)malloc(strlen(dir_path)+strlen(filename)+1);
    cur_path[0]='\0';
    strcat(cur_path,dir_path);
    strcat(cur_path,filename);
-
-	
-//	printf("Dir: %s\n",cur_path);
+   
    if (tmp_path != NULL) {
       ptr1=cur_path;
       ptr2=tmp_path;
-		
+      
       for (i=0; ptr1[0] == ptr2[0] && (ptr1[0] != '\0' && ptr2[0] != '\0') ; i++) {
-			ptr1++;
-			ptr2++;
-		}
-/*		printf("I: %d\n",i);
+         ptr1++;
+         ptr2++;
+      }
+/*      printf("I: %d\n",i);
 
-		printf("TMP: %s\n",tmp_path);
-		printf("CUR: %s\n",cur_path);
-		if (i < t_num)
-			 fr_num = (strlen(tmp_path) - i) * (-1);
-		else */
-		
-		fr_num = i - t_num;
-			
-		t_num = t_num + fr_num;				
-		
+      printf("TMP: %s\n",tmp_path);
+      printf("CUR: %s\n",cur_path);
+      if (i < t_num)
+          fr_num = (strlen(tmp_path) - i) * (-1);
+      else */
+      
+      fr_num = i - t_num;
+         
+      t_num = t_num + fr_num;            
+      
       ptr2 = strchr(ptr1,'\0');
       new_path=(char *)malloc((ptr2-ptr1)+1);
       new_path[0] = '\0';
-      strncat(new_path,ptr1,ptr2-ptr1);	      
-//		printf("NEW: %s\n",new_path);
+      strncat(new_path,ptr1,ptr2-ptr1);         
+
    } else {
       new_path = (char *)malloc(strlen(cur_path)+1);
       new_path[0] = '\0';
       strcat(new_path,cur_path);
-		t_num = strlen(new_path);
+      t_num = strlen(new_path);
    }
-	
-//	printf("%d %s\n",fr_num,new_path);	
    
    if (tmp_path != NULL)
        free(tmp_path);
-	
-	
+   
+   
    tmp_path = (char *)malloc(strlen(cur_path)+1);
    tmp_path[0] = '\0';
    strcat(tmp_path,cur_path);
-	
-/*	fd = open(TMPSLOCATEDB,O_WRONLY|O_APPEND|O_CREAT,00600);
-	new_fr_num = htons(fr_num);
+   
+   fd = fopen(TMPSLOCATEDB,"a");
+   if (fr_num > -128 && fr_num < 128)
+       putc(fr_num,fd);
+   else if (fr_num > 127 || fr_num < -127) {
+      putc (SLOC_ESC, fd);
+      put_short (fr_num, fd);
+   }
+   fputs(new_path,fd);
+   putc('\0',fd);
 
-	write(fd,(char *)&new_fr_num,sizeof(new_fr_num));		 
-	write(fd,new_path,strlen(new_path));
-	write(fd,"\0",1);
-	close(fd); */
-	
-	fd = fopen(TMPSLOCATEDB,"a");
-	if (fr_num > -128 && fr_num < 128)
-		 putc(fr_num,fd);
-	else if (fr_num > 127 || fr_num < -127) {
-		putc (SLOC_ESC, fd);
-		put_short (fr_num, fd);
-	}
-	fputs(new_path,fd);
-	putc('\0',fd);
-
-	if (slevel == '1') {
-		suid = htons(statres.st_uid);
-		sgid = htons(statres.st_gid);
-		smode = htons(statres.st_mode);	
-		
-		if ((suid == tuid && sgid == tgid && smode == tmode)) {
-			put_short(65535,fd);
-		} else {
-			Uput_short(suid,fd);
-			Uput_short(sgid,fd);	
-			Uput_short(smode,fd);
-		}
-		
-		tuid = suid;
-		tgid = sgid;
-		tmode = smode;		
-	}
-	
-	fclose(fd);
-	
-	if (first) {
-		chmod(TMPSLOCATEDB,00640);
-		chown(TMPSLOCATEDB,UID,SLOC_GID);
-		first=0;
-	}
-	
+   fclose(fd);
+   
+   if (first) {
+      chmod(TMPSLOCATEDB,00640);
+      chown(TMPSLOCATEDB,UID,SLOC_GID);
+      first=0;
+   }
+   
    free(new_path);
    free(cur_path);
    
@@ -553,42 +499,47 @@ create_db(char *dirstr)
    dir_item *tmpptr=NULL;
    int load;
    int jump=0;
-	int list_down=0;
-	FILE *fd;
+   int list_down=0;
+   FILE *fd;
 
-	fd = fopen(TMPSLOCATEDB,"w");
-	putc(slevel,fd);
-	fclose(fd);
-	
-	if (dirstr) {
-		/* Make sure there are no problems accessing the source directory */
-		if (lstat(dirstr,&statres) == -1) {
-			fprintf(stderr,"%s: lstat: d%s/: %s\n",progname,dirstr,strerror(errno));
-			return(-1);
-		}
-	}
-	
+   if (UID != SLOC_UID) {
+      fprintf(stderr,"%s: You are not authorized to create an slocate database!\n",progname);
+      return(0);
+   }
+   
+   fd = fopen(TMPSLOCATEDB,"w");
+   putc(slevel,fd);
+   fclose(fd);
+   
+   if (dirstr) {
+      /* Make sure there are no problems accessing the source directory */
+      if (lstat(dirstr,&statres) == -1) {
+         fprintf(stderr,"%s: lstat: d%s/: %s\n",progname,dirstr,strerror(errno));
+         return(-1);
+      }
+   }
+   
    /* Initialize 2D Linked List */
    listptr = init_2D_list();
 
-	if (dirstr) {
-		 if (dirstr[strlen(dirstr)-1] == '/') dirstr[strlen(dirstr)-1] = '\0';
-	} else {
-		dirstr = (char *)malloc(2);
-		dirstr[0]='/';
-		dirstr[1]='\0';
-	}
+   if (dirstr) {
+       if (dirstr[strlen(dirstr)-1] == '/') dirstr[strlen(dirstr)-1] = '\0';
+   } else {
+      dirstr = (char *)malloc(2);
+      dirstr[0]='/';
+      dirstr[1]='\0';
+   }
 
-	
+   
    listptr->name = (char *)statmalloc(strlen(dirstr)+1);
    listptr->name[0] = '\0';
-	strcat(listptr->name,dirstr);
+   strcat(listptr->name,dirstr);
    listptr->st_uid = statres.st_uid;
    listptr->st_gid = statres.st_gid;
    listptr->st_mode = statres.st_mode;
-	listptr->root = 1;
-	listptr->empty=0;
-	
+   listptr->root = 1;
+   listptr->empty=0;
+   
    
    /* Open Source Directory */
    if ((dir_fd = open(dirstr,O_RDONLY)) == -1) {
@@ -601,11 +552,11 @@ create_db(char *dirstr)
       close(dir_fd);
       return(0);
    }
-	
-	if (EXCLUDE && match_exclude(dirstr,""))
-		 load = 0;
-	else
-		 load = 1;
+   
+   if (EXCLUDE && match_exclude(dirstr,""))
+       load = 0;
+   else
+       load = 1;
 
    while (!jump) {
       if (load) {
@@ -618,16 +569,16 @@ create_db(char *dirstr)
       if (!listptr->empty) {
 
          tmpptr = listptr;
-			if (list_down) {
-				listptr=listptr->down;
-			}
-			list_down=0;			
+         if (list_down) {
+            listptr=listptr->down;
+         }
+         list_down=0;         
          while (listptr != NULL) {
-				lstat(listptr->name,&statres);
-				if (!EXCLUDE || (EXCLUDE && !match_exclude(get_cur_dir(listptr,dirstr),listptr->name))) {					
-					if (statres.st_uid < 65535)
-						 frcode(get_cur_dir(listptr,dirstr),listptr->name,statres);					
-				}
+            lstat(listptr->name,&statres);
+            if (!EXCLUDE || (EXCLUDE && !match_exclude(get_cur_dir(listptr,dirstr),listptr->name))) {               
+               if (statres.st_uid < 65535)
+                   frcode(get_cur_dir(listptr,dirstr),listptr->name,statres);               
+            }
             if (!listptr->type) {
                listptr=listptr->down;
             } else {
@@ -636,17 +587,17 @@ create_db(char *dirstr)
             }
          }
       } else {
-			if (!MYchdir("..","create_db")) {
-				close(dir_fd);
-				return(1);
-			}			
+         if (!MYchdir("..","create_db")) {
+            close(dir_fd);
+            return(1);
+         }         
          listptr=listptr->left;
          free_right(listptr);
          tmpptr=listptr;
          listptr=listptr->down;
-			if (listptr != NULL)
-				 load = listptr->type;
-			list_down=0;
+         if (listptr != NULL)
+             load = listptr->type;
+         list_down=0;
       }
       
       if (listptr == NULL) {
@@ -682,25 +633,25 @@ create_db(char *dirstr)
          
          /* Change to new directory */
          if ((EXCLUDE && match_exclude(get_cur_dir(listptr,dirstr),listptr->name) == 1) || (chdir(listptr->name) == -1)) {
-				load=0;
-				list_down=1;
+            load=0;
+            list_down=1;
          } else {
-				
-				close(dir_fd);
-				
-				if ((dir_fd = open(".",O_RDONLY)) == -1) {
-					fprintf(stderr,"%s: open: CDB_5: %s\n",progname,strerror(errno));
-					return(1);
-				}
-			}
+            
+            close(dir_fd);
+            
+            if ((dir_fd = open(".",O_RDONLY)) == -1) {
+               fprintf(stderr,"%s: open: CDB_5: %s\n",progname,strerror(errno));
+               return(1);
+            }
+         }
                   
       }
 
    }   
-	
-	rename(TMPSLOCATEDB,SLOCATEDB);
-	chmod(SLOCATEDB,00640);
-	chown(SLOCATEDB,UID,SLOC_GID);	
+   
+   rename(TMPSLOCATEDB,SLOCATEDB);
+   chmod(SLOCATEDB,00640);
+   chown(SLOCATEDB,UID,SLOC_GID);   
    return res;
 }
 
@@ -709,147 +660,119 @@ create_db(char *dirstr)
 int
 decode_db(char *str)
 {
-	int res = 1;
-	FILE *fd;
-	char *tmppath=NULL;
-	char *codedpath=NULL;
-	char *pcodedpath=NULL;
-	short code_num;
-	int codedpathlen=0;
-	char *newpath=NULL;
-	register char ch;
-	int grow;
-	int jump=0;
-	int tmpgrow=0;
-	int printit;
-	unsigned short suid=65535;
-	unsigned short sgid=65535;
-	unsigned short smode=65535;
-	char *modestr;
-	
-	if ((fd = fopen(SLOCATEDB,"r")) == NULL) {
-		fprintf(stderr,"%s: decode_db(): open: (%d) %s\n",progname,errno,strerror(errno));
-		return(0);
-	}
-	codedpath = (char *)malloc(1);
+   int res = 1;
+   FILE *fd;
+   char *tmppath=NULL;
+   char *codedpath=NULL;
+   char *pcodedpath=NULL;
+   short code_num;
+   int codedpathlen=0;
+   char *newpath=NULL;
+   register char ch;
+   int grow;
+   int jump=0;
+   int tmpgrow=0;
+   int printit;
+   unsigned short smode=65535;
+   char *modestr;
+   struct stat statres;
+  
+   if ((fd = fopen(SLOCATEDB,"r")) == NULL) {
+      fprintf(stderr,"%s: decode_db(): open: (%d) %s\n",progname,errno,strerror(errno));
+      return(0);
+   }
+   codedpath = (char *)malloc(1);
 
-	slevel = getc(fd);
-	
-//	while (read(fd,(char *)&code_num,sizeof(code_num)) > 0) {
-	while ((code_num = getc(fd)) != EOF) {
-//		printf("Here: %d %d\n",code_num,EOF);
-		if (code_num == SLOC_ESC) {
-			code_num = get_short(fd);			
-		} else if (code_num > 127 && code_num < 256)
-			 code_num = code_num - 256;
-		
-//		printf("Code: %d\n",code_num);
+   slevel = getc(fd);
+   
+   while ((code_num = getc(fd)) != EOF) {
+      if (code_num == SLOC_ESC) {
+         code_num = get_short(fd);         
+      } else if (code_num > 127 && code_num < 256)
+          code_num = code_num - 256;
+      
+      free(codedpath);
+      codedpath = (char *)malloc(1);
+      grow=0;
+      jump=0;
+      while(!jump) {
+         ch = getc(fd);
+         grow++;
+         codedpath = (char *)realloc(codedpath,grow);
+         codedpath[grow-1] = ch;
+         if (ch == '\0') jump=1;
+      }
 
-//	   printf("%d ",code_num);
-		free(codedpath);
-		codedpath = (char *)malloc(1);
-		grow=0;
-		jump=0;
-		while(!jump) {
-//			read(fd,(char *)&ch,1);
-			ch = getc(fd);
-			grow++;
-			codedpath = (char *)realloc(codedpath,grow);
-			codedpath[grow-1] = ch;
-			if (ch == '\0') jump=1;
-		}
-//		printf("New: %s\n",codedpath);
-		codedpathlen=grow-1;
-//		printf("%d %s\n",code_num,codedpath);
+      codedpathlen=grow-1;
+      
+      if (tmppath == NULL) {
+         
+         newpath = (char *)malloc(codedpathlen+1);
+         newpath[0] = '\0';
+         strcat(newpath,codedpath);
+         
+         pcodedpath = (char *)malloc(codedpathlen+1);
+         pcodedpath[0] = '\0';
+         strcat(pcodedpath,codedpath);
+         
+         tmpgrow+=codedpathlen;         
+         tmppath = (char *)malloc(tmpgrow+1);
+         tmppath[0] = '\0';
+         strcat(tmppath,codedpath);
+         
+      } else {
 
-		if (slevel == '1') {
-			suid = Uget_short(fd);
-			suid = ntohs(suid);
-			if (suid == 65535) {
-				suid = tuid;
-				sgid = tgid;
-				smode = tmode;
-			} else { 
-				sgid = Uget_short(fd);
-				sgid = ntohs(sgid);
-				smode = Uget_short(fd);
-				smode = ntohs(smode);
-			}
-		
-			tuid = suid;
-			tgid = sgid;
-			tmode = smode;
-		}
-		
-		if (tmppath == NULL) {
-			
-			newpath = (char *)malloc(codedpathlen+1);
-			newpath[0] = '\0';
-			strcat(newpath,codedpath);
-			
-			pcodedpath = (char *)malloc(codedpathlen+1);
-			pcodedpath[0] = '\0';
-			strcat(pcodedpath,codedpath);
-			
-			tmpgrow+=codedpathlen;			
-			tmppath = (char *)malloc(tmpgrow+1);
-			tmppath[0] = '\0';
-			strcat(tmppath,codedpath);
-			
-		} else {
+         tmpgrow+=code_num;
+         if (code_num != 0)
+             tmppath = (char *)realloc(tmppath,tmpgrow+1);
+         if (code_num > 0) {
+            strncat(tmppath,pcodedpath,code_num);
+         } else if (code_num < 0) {
+            tmppath[tmpgrow] = '\0';
+         }
 
-			tmpgrow+=code_num;
-			if (code_num != 0)
-				 tmppath = (char *)realloc(tmppath,tmpgrow+1);
-			if (code_num > 0) {
-				strncat(tmppath,pcodedpath,code_num);
-			} else if (code_num < 0) {
-				tmppath[tmpgrow] = '\0';
-			}
-//			printf("TMP: %d: %s\n",code_num,tmppath);
-//			printf("PTM: %s\n",pcodedpath);
-//			printf("COD: %s\n",codedpath);
-			pcodedpath = (char *)malloc(codedpathlen+1);
-			pcodedpath[0] = '\0';
-			strcat(pcodedpath,codedpath);
-			
-//			printf("NEW: %s\n",newpath);
-			free(newpath);
-			newpath = (char *)malloc(tmpgrow+codedpathlen+1);
-			newpath[0] = '\0';
-			strcat(newpath,tmppath);
-			strcat(newpath,codedpath);
-		}
-		if (strstr(newpath,str) != NULL) {
-			if (slevel == '1') {
-				printit=0;			
-				if (UID == 0)
-					 printit=1;		  
-				else {
-					smode &= 0777;
-					modestr = (char *)malloc(4);
-					snprintf(modestr,4,"%o\0",smode);
-					
-					if (modestr[2] > '3') {
-						printit=1;
-					} else if (GID == sgid && modestr[1] > '3') {
-						printit=1;
-					} else if (UID == suid && modestr[0] > '3') {
-						printit=1;
-					}
-					
-					free(modestr);
-				}
-			} else
-				 printit=1;
-			if (printit)
-				 printf("%s\n",newpath);
-		}
-	}		
-	
-	fclose(fd);  
-	
-	return(res);
+         pcodedpath = (char *)malloc(codedpathlen+1);
+         pcodedpath[0] = '\0';
+         strcat(pcodedpath,codedpath);
+
+         free(newpath);
+         newpath = (char *)malloc(tmpgrow+codedpathlen+1);
+         newpath[0] = '\0';
+         strcat(newpath,tmppath);
+         strcat(newpath,codedpath);
+      }
+      if (strstr(newpath,str) != NULL) {
+         if (slevel == '1') {
+            lstat(newpath,&statres);
+            printit=0;
+            if (UID == 0)
+                printit=1;        
+            else {
+               smode = statres.st_mode & 0777;
+               modestr = (char *)malloc(4);
+               snprintf(modestr,4,"%o\0",smode);
+               
+               if (modestr[2] > '3') {
+                  printit=1;
+               } else if (GID == statres.st_gid && modestr[1] > '3') {
+                  printit=1;
+               } else if (UID == statres.st_uid && modestr[0] > '3') {
+                  printit=1;
+               }
+               
+               free(modestr);
+            }
+         } else
+             printit=1;
+         
+         if (printit)
+             printf("%s\n",newpath);
+      }
+   }      
+   
+   fclose(fd);  
+   
+   return(res);
 }
 
 /* Main Function */
@@ -862,54 +785,63 @@ main(int args, char **argv)
    extern char *optarg;
    extern int optind, opterr, optopt;
    char *p;
-	int SPECDIR=0;
-	int ROOTDIR=0;	
-	char *spec_dir=NULL;
-	int optlen;
+   int SPECDIR=0;
+   int ROOTDIR=0;   
+   char *spec_dir=NULL;
+   int optlen;
 
    progname = ((p = rindex(argv[0],'/')) ? p+1 : *argv);   
    
-	if (args < 2) 
-		 usage();
-	
-	UID = getuid();
-	GID = getgid();		
-	
+   if (args < 2) 
+       usage();
+   
+   UID = getuid();        
+   GID = getgid();      
+   
    while ((ch = getopt(args,argv,"VuU:e:l:")) != EOF) {
       switch(ch) {
       case 'V':
          printf("%s",VERSION);
+         exit(1);
          break;
-		case 'e':
-			EXCLUDE_DIR = parse_exclude(optarg);
-			break;
-		case 'l':
-			slevel = optarg[0];
-			if (slevel != '0' && slevel != '1') {
-				fprintf(stderr,"%s: ERROR: Security level must be 0 or 1.\n",progname);
-				exit(1);
-			}
-		case 'u':
-			ROOTDIR=1;			
-			break;
-		case 'U':
-			SPECDIR=1;
-			optlen=strlen(optarg);
-			spec_dir = (char *)malloc(optlen+1);
-			memmove(spec_dir,optarg,optlen);
-			spec_dir[optlen] = '\0';			
-			break;
+      case 'e':
+         EXCLUDE_DIR = parse_exclude(optarg);
+         break;
+      case 'l':
+         slevel = optarg[0];
+         if (slevel != '0' && slevel != '1') {
+            fprintf(stderr,"%s: ERROR: Security level must be 0 or 1.\n",progname);
+            exit(1);
+         }
+         break;
+      case 'u':
+         ROOTDIR=1;         
+         break;
+      case 'U':
+         SPECDIR=1;
+         optlen=strlen(optarg);
+         spec_dir = (char *)malloc(optlen+1);
+         memmove(spec_dir,optarg,optlen);
+         spec_dir[optlen] = '\0';         
+         break;
+      default:
+         return(1);
+         break;
+         
       }
    }
-	
-	SLOC_GID = get_gid(GRPFILE);
-	
-	if (SPECDIR)		
-		res = create_db(spec_dir);
-	else if (ROOTDIR)
-		 res = create_db((char *)NULL);
-	else if (argv[1])
-		 res = decode_db(argv[1]);
+   
+   SLOC_GID = get_gid(GRPFILE);
+   
+   if (SPECDIR) {
+      res = create_db(spec_dir);
+   }
+   else if (ROOTDIR)
+      res = create_db((char *)NULL);
+   else if (argv[optind]) {
+      res = decode_db(argv[1]);
+   } else
+       usage();
 
    if (!res)
        return(1);
